--[[
 MiniPet Addon
 Version 4.0
 $Revision: 77708 $

 Currently hosted by Breanni of Scarlet Crusade
 All credit for authoring this addon is listed below

 Visit WarcraftPets.com for all your minipet needs!
]]--

--[[
    MiniPet by Carra of Stormrage

    There are a number of events where the player looses his pet and we can notify him:
    -Entering the game                   >OK
    -Entering an instance                >OK
    -Dieing and getting back alive       >Usually OK :p
    -Using a Taxi Service                >OK
    -Teleporting/Summoning/Hearthstoning >OK but no summoning

    Based originally on MiniPetLeash (completely rewritten except for the events which are copied).

    Updated and converted to Ace3 by LordFarlander
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "MiniPet" );

local BAG_TYPE_VANITYPET = 4096;

_G["BINDING_NAME_CLICK " .. L["MiniPet"] .."_Button:LeftButton"] = L["Summon Minipet"];
BINDING_HEADER_MINIPETHEADER = L["MiniPet"];

if( not LibStub( "LibBabble-Zone-3.0", true ) ) then
    error( L["%s requires %s"]:format( L["MiniPet"], "LibBabble-Zone-3.0" ) );
    return;
end--if

if( not LibStub( "LibSpecialEvents-Mount-4.0", true ) ) then
    error( L["%s requires %s"]:format( L["MiniPet"], "LibSpecialEvents-Mount-4.0" ) );
    return;
end--if

local LibLordFarlander = LibStub( "LibLordFarlander-2.0", true );
if( not LibLordFarlander ) then
    error( L["%s requires %s"]:format( L["MiniPet"], "LibLordFarlander-2.0" ) );
    return;
end--if

local ItemList = LibStub( "LibLordFarlander-ItemList-2.0", true );
if( not ItemList ) then
    error( L["%s requires %s"]:format( L["MiniPet"], "LibLordFarlander-ItemList-2.0" ) );
    return;
end--if

if( not LibStub( "LibLordFarlander-UI-2.0", true ) ) then
    error( L["%s requires %s"]:format( L["MiniPet"], "LibLordFarlander-UI-2.0" ) );
    return;
end--if

if( not LibStub( "LibPetAndMountDatabase-1.1", true ) ) then
    error( L["%s requires %s"]:format( L["MiniPet"], "LibPetAndMountDatabase-1.1" ) );
    return;
end--if

MiniPet = LibStub( "AceAddon-3.0" ):NewAddon( L["MiniPet"], "AceEvent-3.0", "AceTimer-3.0", "AceConsole-3.0" );
MiniPet.version = tonumber( string.match( "$Rev: 77708 $", "(%d+)" ) );

--Global Values
MiniPet.DEBUG = false;            --true = Debugging Mode
MiniPet.dprint = LibLordFarlander.dprint;
MiniPet.button = nil;

MiniPet.UpdateInterval      = 0.2;   --Time between two flashes of the button in seconds
MiniPet.TotalFlashTime      = 10;    --Total time to flash the button in seconds

--DO NOT CHANGE THESE VALUES
MiniPet.PlayersPets         = {};    --List of the players pet
MiniPet.CurrentPet          = nil;   --Pet on the Button
MiniPet.BoardedTaxi         = false;
MiniPet.StartedHearthstone  = false;
MiniPet.HearthstoneComplete = false;
MiniPet.PlayerDied          = false;
MiniPet.PlayerEnteringWorld = true;
MiniPet.ReadyToInitialize   = false;
MiniPet.PetListOkay         = false;
MiniPet.LastPet             = nil;
MiniPet.QuestPet            = nil;

MiniPet.ShowButtonImg       = false; --Goes on and off if button flashes
MiniPet.FlashTime           = 0;     --Time since button flashes
MiniPet.FlashButton         = false; --On when the button flashes

MiniPet.Dirty               = false;

MiniPet.keyBoundMode        = false;

MiniPet.ui = { currentZone = "", currentFavorite = 0, };

local MINIPET_BATTLEGROUNDS = nil

function MiniPet:OnInitialize()
    local AceConfigReg = LibStub( "AceConfigRegistry-3.0" );
    local AceConfigDialog = LibStub( "AceConfigDialog-3.0" );
    local defaults = {
            profile = {
                BattleGroundCheck = true,
                GiveTextWarning = true,
                GiveButtonWarning = true,
                GiveMountWarning = true,
                SuggestOnlySoulbound = true,
                ReagentCheck = true,
                EquipmentCheck = true,
                ForceQuestPet = true,
                -- List of favorite pets, these will be picked more often
                -- stored as [itemID]=true
                Favorites = {},
                -- List of pets per zone, if the pets are in the player's bags they will be picked
                -- when in the indicated zone
                -- stored as ["Localized zone name"]=itemID
                ZoneFavorites = {},
                Button = LibStub( "LibLordFarlander-UI-2.0" ).GetButtonSettingsDefaults(),
            },
        };
    local buttonOptions = {
        type = "group",
        handler = self,
        name = L["Button"],
        desc = L["Options relating to the button."],
        args = LibStub( "LibLordFarlander-UI-2.0" ).GetButtonOptionArgs( 1, {
            toggleButton = L["toggleButton"],
            ["Show button"] = L["Show button"],
            ["Show/hide the button."] = L["Show/hide the button."],
            autoHideButton = L["autoHideButton"],
            ["Automatically hide button"] = L["Automatically hide button"],
            ["Hide the button automatically?"] = L["Hide the button when you click it?"],
            lockButton = L["lockButton"],
            ["Lock button position"] = L["Lock button position"],
            ["Lock/unlock the position of the button."] = L["Lock/unlock the position of the button."],
            ["Alpha of button"] = L["Alpha of button"],
            ["Sets the alpha value that the button has normally."] = L["Sets the alpha value that the button has normally."],
            fadeOut = L["fadeOut"],
            ["Fade out"] = L["Fade out"],
            ["Should the button fade out?"] = L["Fade the button out when the cursor is not over it?"],
            fadeOutInsteadHide = L["fadeOutInsteadHide"],
            ["Fade out instead of hiding"] = L["Fade out instead of hiding"],
            ["Should the button fade out instead of hiding?"] = L["Fade the button out instead of hiding it?"],
            fadeOutCancelInCombat = L["fadeOutCancelInCombat"],
            ["Cancel fade out if in combat"] = L["Cancel fade out if in combat"],
            ["Should the button not fade out if you're in combat?"] = L["Should the button not fade out if you're in combat?"],
            ["Cancel fade out if holding Shift"] = L["Cancel fade out if holding Shift"],
            ["Should the button not fade out if you're holding Shift?"] = L["Should the button not fade out if you're holding Shift?"],
            ["Cancel fade out if holding Ctrl"] = L["Cancel fade out if holding Ctrl"],
            ["Should the button not fade out if you\re holding Ctrl?"] = L["Should the button not fade out if you're holding Ctrl?"],
            ["Cancel fade out if holding Alt"] = L["Cancel fade out if holding Alt"],
            ["Should the button not fade out if you're holding Alt?"] = L["Should the button not fade out if you're holding Alt?"],
            ["Fade out time"] = L["Fade out time"],
            ["Sets how long the button takes to fade out."] = L["Sets how long the button takes to fade out."],
            ["Fade out delay"] = L["Fade out delay"],
            ["Sets the delay before the button starts to fade."] = L["Sets the delay before the button starts to fade."],
            ["Alpha after fadeout"] = L["Alpha after fadeout"],
            ["Sets the alpha value that the button has after faded out."] = L["Sets the alpha value that the button has after faded out."],
            } ),
    };
    local favoritesOptions = {
        type = "group",
        name = L["Favorites"],
        desc = L["Minipets that should be picked more often if you have them."],
        handler = self,
        args = {
            favorites = {
                type = "select",
                order = 1,
                name = L["Favorites"],
                desc = L["Minipets that should be picked more often if you have them."],
                width = "full",
                values = function( info )
                             local Favorites = {};
                             local firstItem = 0;

                             for itemID, _ in pairs( info.handler.db.profile.Favorites ) do
                                 local sName, _, _, _, _, _, _, _, _, sItemTexture = GetItemInfo( itemID );

                                 if( not sName ) then
                                     sName = tostring( ( itemID < 0 ) and ( itemID * -1 ) or itemID );
                                 end--if
                                 if( sItemTexture ) then
                                     Favorites[itemID] = ("|T%s:0|t %s"):format( sItemTexture, sName );
                                 else
                                     Favorites[itemID] = sName;
                                 end--if
                                 if( firstItem == 0 ) then
                                     firstItem = itemID;
                                 end--if
                             end--for
                             if( ( not info.handler.db.profile.Favorites[info.handler.ui.currentFavorite] ) or ( info.handler.ui.currentFavorite == 0 ) ) then
                                 info.handler.ui.currentFavorite = firstItem;
                             end--if
                             return Favorites;
                         end,
                get = function( info )
                          return info.handler.ui.currentFavorite;
                      end,
                set = function( info, val )
                          info.handler.ui.currentFavorite = val;
                      end,
            },
            minipet = {
                type = "input",
                order = 2,
                name = L["MiniPet"],
                desc = L["Enter the name or ID of a minipet to add to the list then press Enter or click Okay."],
                width = "full",
                get = function( info )
                          return info.handler.db.profile.Favorites[info.handler.ui.currentFavorite] or "";
                      end,
                set = function( info, val )
                          --Assume was validated
                          local itemID = LibLordFarlander.GetItemIDFromLink( LibLordFarlander.GetItemLink( val ) );

                          info.handler.db.profile.Favorites[itemID] = true;
                          info.handler.ui.currentFavorite = itemID;
                      end,
                validate = function( info, val )
                               local itemLink = LibLordFarlander.GetItemLink( val );

                               if( itemLink and LibStub( "LibPetAndMountDatabase-1.1" ):ItemInSet( itemLink, "MiniPets" ) ) then
                                   return true;
                               end--if
                               if( type( val ) == "number" ) then
                                   return L["Cannot find a minipet with itemID %i!"]:format( val );
                               else
                                   return L["Cannot find a minipet named %q!"]:format( val );
                               end--if
                               return false;
                           end,
            },
            delete = {
                type = "execute",
                order = 3,
                name = L["Delete"],
                desc = L["Delete the currently selected minipet from the list."],
                func = function( info )
                           info.handler.db.profile.Favorites[info.handler.ui.currentFavorite] = nil;
                           info.handler.ui.currentFavorite = 0;
                       end,
            },
        },
    };
    local zoneOptions = {
        type = "group",
        name = L["Zone Favorites"],
        desc = L["Set a minipet to use when in a certain zone."],
        handler = self,
        args = {
            zones = {
                type = "select",
                order = 1,
                name = L["Zones"],
                desc = L["Set a minipet to use when in a certain zone."],
                width = "full",
                values = function( info )
                             local zones = {};

                             for continent, _ in pairs( { GetMapContinents() } ) do
                                 local continentZones = { GetMapZones( continent ) };

                                 for _, zone in pairs( continentZones ) do
                                     local sName, sItemTexture = L["none"], nil;

                                     if( info.handler.db.profile.ZoneFavorites[zone] ) then
                                         sName, _, _, _, _, _, _, _, _, sItemTexture = GetItemInfo( info.handler.db.profile.ZoneFavorites[zone] );
                                     end--if
                                     if( not sName ) then
                                         sName = L["none"];
                                         info.handler.db.profile.ZoneFavorites[zone] = nil;
                                     end--if
                                     if( sItemTexture ) then
                                         zones[zone] = ("%s [|T%s:0|t %s]"):format( zone, sItemTexture, sName );
                                     else
                                         zones[zone] = ("%s [%s]"):format( zone, sName );
                                     end--if
                                 end--for
                             end--for
                             if( #zones ) then
                                 if( info.handler.ui.currentZone == "" ) then
                                     info.handler.ui.currentZone = GetRealZoneText();
                                 end--if
                             else
                                 info.handler.ui.currentZone = "";
                             end--if                                     
                             return zones;
                         end,
                get = function( info )
                          return info.handler.ui.currentZone;
                      end,
                set = function( info, val )
                          info.handler.ui.currentZone = val;
                      end,
            },
            minipet = {
                type = "input",
                order = 2,
                name = L["MiniPet"],
                desc = L["Enter the name or ID of a minipet to add to the list then press Enter or click Okay."],
                width = "full",
                get = function( info )
                          return info.handler.db.profile.ZoneFavorites[info.handler.ui.currentZone] or "";
                      end,
                set = function( info, val )
                          local itemLink = LibLordFarlander.GetItemLink( val );
                          
                          if( itemLink and LibStub( "LibPetAndMountDatabase-1.1" ):ItemInSet( itemLink, "MiniPets" ) ) then
                              info.handler.db.profile.ZoneFavorites[info.handler.ui.currentZone] = LibLordFarlander.GetItemIDFromLink( LibLordFarlander.GetItemLink( val ) );
                          else
                              info.handler.db.profile.ZoneFavorites[info.handler.ui.currentZone] = nil;
                          end--if
                          MiniPet:GetPlayersPets();
                          MiniPet:ChooseRandomPet();
                      end,
                validate = function( info, val )
                               local itemLink = LibLordFarlander.GetItemLink( val );

                               if( ( val == "" ) or ( itemLink and LibStub( "LibPetAndMountDatabase-1.1" ):ItemInSet( itemLink, "MiniPets" ) ) ) then
                                   return true;
                               end--if
                               if( type( val ) == "number" ) then
                                   return L["Cannot find a minipet with itemID %i!"]:format( val );
                               else
                                   return L["Cannot find a minipet named %q!"]:format( val );
                               end--if
                               return false;
                           end,
            },
        },
    };
    local options = {
        type = "group",
        handler = self,
        args = {
            debug = {
                type = "toggle",
                hidden = true,
                name = "Debug",
                desc = "Debug mode toggle",
                get = function( info )
                          return MiniPet.DEBUG;
                      end,
                set = function( info, val )
                          MiniPet.DEBUG = not MiniPet.DEBUG;
                      end,
            },
            [L["toggleTextWarning"]] = {
                type = "toggle",
                order = 1,
                name = L["Text warning"],
                desc = L["Show text warnings to alert you to summon your pet."],
                get = function( info )
                          return MiniPet.db.profile.GiveTextWarning;
                      end,
                set = function( info, val )
                          MiniPet.db.profile.GiveTextWarning = not MiniPet.db.profile.GiveTextWarning;
                      end,
            },
            [L["toggleButtonWarning"]] = {
                type = "toggle",
                order = 2,
                name = L["Button warning"],
                desc = L["Flash the button to alert you to summon your pet."],
                get = function( info )
                          return MiniPet.db.profile.GiveButtonWarning;
                      end,
                set = function( info, val )
                          MiniPet.db.profile.GiveButtonWarning = not MiniPet.db.profile.GiveButtonWarning;
                      end,
            },
            [L["toggleMountWarning"]] = {
                type = "toggle",
                order = 3,
                name = L["Mount warning"],
                desc = L["Show an alert when you dismount from your flying mount."],
                get = function( info )
                          return self.db.profile.GiveMountWarning;
                      end,
                set = function( info, val )
                          MiniPet.db.profile.GiveMountWarning = not MiniPet.db.profile.GiveMountWarning;
                      end,
            },
            [L["toggleBG"]] = {
                type = "toggle",
                order = 4,
                name = L["Battleground check"],
                desc = L["Show minipet alerts while in a battleground."],
                get = function( info )
                          return MiniPet.db.profile.BattleGroundCheck;
                      end,
                set = function( info, val )
                          MiniPet.db.profile.BattleGroundCheck = not MiniPet.db.profile.BattleGroundCheck;
                      end,
            },
            [L["suggestOnlySoulbound"]] = {
                type = "toggle",
                order = 5,
                name = L["Suggest only soulbound minipets"],
                desc = L["Toggles whether only soulbound pets will be suggested for use."],
                get = function( info )
                          return MiniPet.db.profile.SuggestOnlySoulbound;
                      end,
                set = function( info, val )
                          MiniPet.db.profile.SuggestOnlySoulbound = not MiniPet.db.profile.SuggestOnlySoulbound;
                          MiniPet:GetPlayersPets();
                          MiniPet:ChooseRandomPet();
                      end,
            },
            [L["reagentCheck"]] = {
                type = "toggle",
                order = 6,
                name = L["Check for reagented minipets that are missing their reagent"],
                desc = L["Toggles whether pets that require a reagent will be suggested when you have none of the required reagent."],
                get = function( info )
                          return MiniPet.db.profile.ReagentCheck;
                      end,
                set = function( info, val )
                          MiniPet.db.profile.ReagentCheck = not MiniPet.db.profile.ReagentCheck;
                          MiniPet:GetPlayersPets();
                          MiniPet:ChooseRandomPet();
                      end,
            },
            [L["equipmentCheck"]] = {
                type = "toggle",
                order = 7,
                name = L["Check to make sure that equipment minipets are equipped"],
                desc = L["Toggles whether pets summoned by equipment will be suggested when the item is unequipped."],
                get = function( info )
                          return MiniPet.db.profile.EquipmentCheck;
                      end,
                set = function( info, val )
                          MiniPet.db.profile.EquipmentCheck = not MiniPet.db.profile.EquipmentCheck;
                          MiniPet:GetPlayersPets();
                          MiniPet:ChooseRandomPet();
                      end,
            },
            [L["forceQuestPet"]] = {
                type = "toggle",
                order = 8,
                name = L["Force to pick a quest minipet if availiable"],
                desc = L["Toggles whether pets that are part of a quest are preferred instead of a random pet."],
                get = function( info )
                          return MiniPet.db.profile.ForceQuestPet;
                      end,
                set = function( info, val )
                          MiniPet.db.profile.ForceQuestPet = not MiniPet.db.profile.ForceQuestPet;
                          MiniPet:GetPlayersPets();
                          MiniPet:ChooseRandomPet();
                      end,
            },
            [L["list"]] = {
                type = "execute",
                order = 9,
                name = L["List all minipets"],
                desc = L["Show a list with all of your minipets."],
                func = "ListPets",
            },
            [L["relist"]] = {
                type = "execute",
                order = 10,
                name = L["Refresh minipet list"],
                desc = L["Force a refresh of the list of all your pets."],
                func = "GetPlayersPets",
            },
        },
        name = L["MiniPet"],
        desc = L["Always keep your little companion with you."],
        icon = "Interface\\AddOns\\MiniPet\\Img\\MiniPet",
    };
    self.db = LibStub( "AceDB-3.0" ):New( "MiniPetDB", defaults );
    self.db.RegisterCallback( self, "OnProfileChanged" );
	LibStub( "AceConfig-3.0" ):RegisterOptionsTable( L["MiniPet"], options, { "minipet", "mp" } );
    AceConfigReg:RegisterOptionsTable( L["MiniPet Button"], buttonOptions );
    AceConfigReg:RegisterOptionsTable( L["MiniPet Zone Favorites"], zoneOptions );
    AceConfigReg:RegisterOptionsTable( L["MiniPet Favorites"], favoritesOptions );
    AceConfigReg:RegisterOptionsTable( L["MiniPet Profile"], LibStub( "AceDBOptions-3.0" ):GetOptionsTable( self.db ) );
    AceConfigDialog:AddToBlizOptions( L["MiniPet"] );
    AceConfigDialog:AddToBlizOptions( L["MiniPet Button"], L["Button"], L["MiniPet"] );
    AceConfigDialog:AddToBlizOptions( L["MiniPet Zone Favorites"], L["Zone Favorites"], L["MiniPet"] );
    AceConfigDialog:AddToBlizOptions( L["MiniPet Favorites"], L["Favorites"], L["MiniPet"] );
    AceConfigDialog:AddToBlizOptions( L["MiniPet Profile"], L["Profile"], L["MiniPet"] );
end--MiniPet:OnInitialize()

function MiniPet:OnProfileReset()
    self:OnProfileChanged();
end--MiniPet:OnProfileReset()

function MiniPet:OnProfileChanged()
    LibStub( "LibLordFarlander-UI-2.0" ).OnProfileChanged( self );
    self:GetPlayersPets();
    self:ChooseRandomPet();
end--MiniPet:OnProfileChanged()

function MiniPet:OnEnable()
    local LibUI = LibStub( "LibLordFarlander-UI-2.0" );

    --Register the Events
    self:RegisterEvent( "BAG_UPDATE",
        function()
            MiniPet.Dirty = true;
            if( not MiniPet.button:IsVisible() ) then
                MiniPet:ScheduleTimer( "OnUpdate", 0 );
            end--if
        end );
    self:RegisterEvent( "ZONE_CHANGED_NEW_AREA",
        function()
            if( MiniPet:HasMiniPetListChanged() ) then
                MiniPet:ChooseRandomPet();
            end--if
        end );
    self:RegisterEvent( "PLAYER_ENTERING_WORLD",
        function()
            MiniPet.PlayerDied = UnitIsDeadOrGhost( "player" );
            MiniPet.BoardedTaxi = UnitOnTaxi( "player" );
            MiniPet.PlayerEnteringWorld = false;
            MiniPet:WarnPlayer();
        end );
    self:RegisterEvent( "TAXIMAP_CLOSED",
        function()
            MiniPet.BoardedTaxi = true;
        end );
    self:RegisterEvent( "PLAYER_CONTROL_GAINED",
        function()
            if( MiniPet.BoardedTaxi ) then
                MiniPet.BoardedTaxi = false;
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "PLAYER_DEAD",
        function()
            MiniPet:dprint( "MiniPet: PlayerDied" );
            MiniPet.PlayerDied = true;
        end );
    self:RegisterEvent( "PLAYER_ALIVE",
        function()
            MiniPet.BoardedTaxi = UnitOnTaxi( "player" );
            if( MiniPet.HearthstoneComplete ) then
                MiniPet.HearthstoneComplete = false;
                MiniPet:dprint( "MiniPet: Activating Pet After Hearthstone!" );
                MiniPet:WarnPlayer();
            end--if
            if( MiniPet.PlayerDied ) then
                MiniPet.PlayerDied = false;
                MiniPet:dprint( "MiniPet: no longer dead!" );
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "PLAYER_UNGHOST",
        function()
            if( MiniPet.PlayerDied ) then
                MiniPet.PlayerDied = false;
                MiniPet:dprint( "MiniPet: no longer ghost!" );
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "UNIT_SPELLCAST_INTERRUPTED",
        function( unit, _, _ )
            if( unit == "player" ) then
                if( MiniPet.StartedHearthstone ) then
                    MiniPet:dprint( "MiniPet: Interrupted Teleporting!" );
                    MiniPet.StartedHearthstone = false;
                end--if
            end--if
        end );
    self:RegisterEvent( "UNIT_SPELLCAST_SUCCEEDED",
        function( unit, spellName, _ )
            if( unit == "player" ) then
                if( MiniPet.StartedHearthstone ) then
                    if( MiniPet.IsATeleportSpell( spellName ) ) then
                        MiniPet:dprint( "MiniPet: Finished Teleporting!" );
                        MiniPet.StartedHearthstone = false;
                        MiniPet.HearthstoneComplete = true;
                        MiniPet:WarnPlayer();
                    end--if
                end--if
            end--if
        end );
    self:RegisterEvent( "UNIT_SPELLCAST_SENT",
        function( unit, spellName, _, _ )
            if( unit == "player" ) then
                MiniPet:dprint( "Spellcast: " .. spellName );
                MiniPet.StartedHearthstone = false;
                if( MiniPet.IsATeleportSpell( spellName ) ) then
                    MiniPet:dprint( "MiniPet: Started Teleporting!" );
                    MiniPet.StartedHearthstone = true;
                end--if
            end--if
        end );
    self:RegisterEvent( "PLAYER_REGEN_ENABLED",
        function()
            MiniPet.button:OutOfCombatReset();
            MiniPet:SetButton( MiniPet.CurrentPet );
        end );

    LibStub("LibSpecialEvents-Mount-4.0").RegisterCallback( self, "Dismounted",
        function( _, _, _, flying )
            if( flying and MiniPet.db.profile.GiveMountWarning ) then
                MiniPet:WarnPlayer();
            end--if
        end );

    LibUI.SetupForUI( self );
    self.button = LibUI.CreateButton( self, L["MiniPet"], "Interface\\AddOns\\MiniPet\\Img\\MiniPet", self.db.profile.ButtonGroup, self.SkinChanged, self );
    ItemList.AttachToButton( self.button );
    self.button:SetMovable( not self.db.profile.Button.LockButton );
    self.button:HookScript( "OnClick",
        function( button )
            button.class.FlashButton = false; --Stop Flashing the Button :)
            if( button.class.db.profile.Button.AutoHideButton ) then
                button:HideEx();
            end--if
            button.class:ChooseRandomPet();
        end );

    --Upgrade check
    if( self.db.profile.LastVersion == nil ) then
        --Upgrading from 2.0 or earlier, import the old settings
        self.db.profile.BattleGroundCheck = MiniPet_BattleGroundCheck;
        self.db.profile.GiveTextWarning = MiniPet_GiveTextWarning;
        self.db.profile.GiveButtonWarning = MiniPet_GiveButtonWarning;
        self.db.profile.Button.AutoHideButton = MiniPet_HideButton;
    end--if
    if( self.db.profile.LastVersion ~= self.version ) then
        self.db.profile.LastVersion = self.version;
    end--if

    self.Dirty = true;
    --Create the Pet Inventory
    self:GetPlayersPets();
    --Set a Pet
    self:ChooseRandomPet();
end--MiniPet:OnEnable();

function MiniPet:OnUpdate( elapsed )
    if( ( UnitName( "player" ) ~= UNKNOWNOBJECT ) and self.ReadyToInitialize ) then
        self:dprint( "MiniPet: PlayerEnteringWorld and ALIVE" );
        self.PlayerEnteringWorld = false;
        self.ReadyToInitialize = false;
        self:WarnPlayer();
    end--if
    
    if( self.Dirty ) then
        self.Dirty = false;
        if( self:HasMiniPetListChanged() ) then
            self:ChooseRandomPet();
        end--if
        self:SetButton( self.CurrentPet );
    end--if

    --Code to let the button flash
    --elapsed = amount of seconds that passed since last update.
    --every updateinterval, change the button image
    --until you reach totalflashtime
    if( self.FlashButton and elapsed ) then
        local mpButton = self.button;

        if( mpButton.TimeSinceLastUpdate == nil ) then
            mpButton.TimeSinceLastUpdate = 0;
            self.FlashTime = 0;
        end--if
        mpButton.TimeSinceLastUpdate = mpButton.TimeSinceLastUpdate + elapsed;
        self.FlashTime = self.FlashTime + elapsed;
        if( self.FlashTime < self.TotalFlashTime ) then
            if( mpButton.TimeSinceLastUpdate > self.UpdateInterval ) then
                if( self.ShowButtonImg and ( self.CurrentPet ~= nil ) ) then
                    mpButton:SetFromListEntry( self.CurrentPet );
                    self.ShowButtonImg = false;
                else
                    mpButton:SetImage( "Interface\\AddOns\\MiniPet\\Img\\MiniPet" );
                    self.ShowButtonImg = true;
                end--if
                mpButton.TimeSinceLastUpdate = 0;
            end--if
        else
            self.FlashButton = false;
            self:SetButton( self.CurrentPet );
            self.FlashTime = 0;
        end--if
    end--if
end--MiniPet:OnUpdate( elapsed )

----------------------------
--+Pet/Inventory Functions--
function MiniPet:GetZonePet( ZoneList )
    if( ZoneList ) then
        local profile = self.db.profile;
        local opReagentCheck = profile.ReagentCheck;
        local opSuggestOnlySoulbound = profile.SuggestOnlySoulbound;

        if( opReagentCheck ) then
            reagented = LibStub( "LibPetAndMountDatabase-1.1" ):ItemInSet( ZoneList, "MiniPets.Reagented" );
        else
            reagented = nil;
        end--if
        if( ( LibLordFarlander.PlayerItemCount( ZoneList ) > 0 ) and LibStub( "LibPetAndMountDatabase-1.1" ):ItemInSet( ZoneList, "MiniPets" ) and ( ( not opReagentCheck ) or ( not reagented ) or ( GetItemCount( reagented ) > 0 ) ) ) then
            local BagItem = ItemList.FindInBags( ZoneList );
            
            if( ( not opSuggestOnlySoulbound ) or LibLordFarlander.ItemIsSoulbound( BagItem.bag, BagItem.slot ) ) then
                return BagItem;
            end--if
        end--if
    end--if
    return nil;
end--MiniPet:GetZonePet( ZoneList )

function MiniPet:GetRandomFromListWithWeighing( set, current, favorites )
    local useset;

    if( ( #favorites > 0 ) and set and ( #set > 1 ) ) then
        local PetAndMountDatabase = LibStub( "LibPetAndMountDatabase-1.1" );
        local profile = self.db.profile;
        local opReagentCheck = profile.ReagentCheck;
        local opSuggestOnlySoulbound = profile.SuggestOnlySoulbound;

        useset = {};
        LibLordFarlander.CopyTable( useset, set );
        for itemID, _ in pairs( favorites ) do
            if( opReagentCheck ) then
                reagented = PetAndMountDatabase:ItemInSet( itemID, "MiniPets.Reagented" );
            else
                reagented = nil;
            end--if
            if( ( LibLordFarlander.PlayerItemCount( itemID ) > 0 ) and PetAndMountDatabase:ItemInSet( itemID, "MiniPets" ) and ( ( not opReagentCheck ) or ( not reagented ) or ( GetItemCount( reagented ) > 0 ) ) ) then
                local BagItem = ItemList.FindInBags( itemID );
                
                if( ( not opSuggestOnlySoulbound ) or LibLordFarlander.ItemIsSoulbound( BagItem.bag, BagItem.slot ) ) then
                    table.insert( useset, BagItem );
                end--if
            end--if
        end--for
    else
        useset = set;
    end--if
    return ItemList.GetRandomItemFromList( useset, current );
end--MiniPet:GetRandomFromListWithWeighing( masterset, subset, current, favorites )

--Pick a random pet and set the button.
function MiniPet:ChooseRandomPet()
    if( self.db.profile.ForceQuestPet and self.QuestPet ) then
        self.CurrentPet = self.QuestPet;
    else
        local profile = self.db.profile;

        self.CurrentPet = self:GetZonePet( profile.ZoneFavorites[GetRealZoneText()] ) or self:GetRandomFromListWithWeighing( self.PlayersPets, self.CurrentPet, profile.Favorites );
    end--if
    self:SetButton( self.CurrentPet );
end--MiniPet:ChooseRandomPet()

--Gets all your pets from your inventory.
function MiniPet:GetPlayersPets()
    self.PlayersPets = {};
    self.QuestPet = nil;

    local PetAndMountDatabase = LibStub( "LibPetAndMountDatabase-1.1" );
    local playerPetList = self.PlayersPets;
    local itemLink;
    local reagented;
    local equipmentSlot;
    local opSuggestOnlySoulbound = self.db.profile.SuggestOnlySoulbound;
    local opReagentCheck = self.db.profile.ReagentCheck;
    local opEquipmentCheck = self.db.profile.EquipmentCheck;

    for bag = 0, NUM_BAG_SLOTS do
        for slot = 1, GetContainerNumSlots( bag ) do
            itemLink = GetContainerItemLink( bag, slot );
            
            if( itemLink ) then
                if( PetAndMountDatabase:ItemInSet( itemLink, "MiniPets" ) ) then
                    if( opReagentCheck ) then
                        reagented = PetAndMountDatabase:ItemInSet( itemLink, "MiniPets.Reagented" );
                    else
                        reagented = nil;
                    end--if
                    if( ( ( not opSuggestOnlySoulbound ) or ( LibLordFarlander.ItemIsSoulbound( bag, slot ) ) ) and
                        ( ( not opReagentCheck ) or ( not reagented ) or ( GetItemCount( reagented ) > 0 ) ) and
                        ( ( not opEquipmentCheck ) or ( not PetAndMountDatabase:ItemInSet( itemLink, "MiniPets.Equipment" ) ) ) ) then
                        ItemList.AddBagItemToList( playerPetList, bag, slot );
                        if( self.db.profile.ForceQuestPet ) then
                            local questID = PetAndMountDatabase:ItemInSet( itemLink, "MiniPets.Quest" );
    
                            if( ( questID and self:HasQuestAndNotComplete( questID ) ) or
                                ( ( self.QuestPet == nil ) and PetAndMountDatabase:ItemInSet( itemLink, "MiniPets.Children" ) ) ) then
                                self.QuestPet = ItemList.BuildBagItem( bag, slot );
                            end--if
                        end--if
                    end--if
                elseif( GetItemFamily( itemLink ) and ( bit.band( GetItemFamily( itemLink ), BAG_TYPE_VANITYPET ) ~= 0 ) ) then
                    -- Assume is an unknown Normal Minipet
                    ItemList.AddBagItemToList( playerPetList, bag, slot );
                end--if
            end--if
        end--for
    end--for
    
    local iterator, handle = PetAndMountDatabase:IterateSet( "MiniPets.Equipment" );
    local pet, equipmentSlot = iterator( handle );

    while pet do
        if( opEquipmentCheck and ( LibLordFarlander.GetItemIDFromLink( GetInventoryItemLink( "player", GetInventorySlotInfo( equipmentSlot ) ) ) == pet ) ) then
            ItemList.AddInventoryItemToList( playerPetList, equipmentSlot );
        end--if
        pet, equipmentSlot = iterator( handle );
    end--for
end--MiniPet:GetPlayerPets()

--Prints out a list with all your pets.
function MiniPet:ListPets()
    self:Print( L["List of your minipets:"] );
    for _, pet in pairs( self.PlayersPets ) do
        self:Print( ("|T%s:0|t %s"):format( pet:GetTexture(), pet:GetLink() ) );
    end--for
end--MiniPet:listPets()
---Pet/Invenroty Functions--
----------------------------

-------------------
--+Help Functions--
function MiniPet.IsATeleportSpell( spellName )
    local iterator, handle = PetAndMountDatabase:IterateSet( "Spells.Teleport" );
    local spell, _ = iterator( handle );

    while spell do
        if( spell < 0 ) then
            spell = -1 * spell;
        end--if
        if( spellName == GetSpellInfo( spell ) ) then
            return true;
        end--if
        spell, _ = iterator( handle );
    end--for
    return false;
end--MiniPet.IsATeleportSpell( spellName )

function MiniPet:HasMiniPetListChanged()
    local prePetCount = #self.PlayersPets;
    local prePetList =  {};

    if( self:GetZonePet( self.db.profile.ZoneFavorites[GetRealZoneText()] ) ) then
        return true;
    end--if

    LibLordFarlander.CopyTable( prePetList, self.PlayersPets );

    self:GetPlayersPets();

    -- differences in sizes always means the list has changed
    if( ( prePetCount ~= #self.PlayersPets ) or ( not self.PetListOkay ) ) then
        self.PetListOkay = true;
        self:dprint( "Number of minipets changed" );
        return true;
    end--if
    self.PetListOkay = true;
    return not LibLordFarlander.CompareTables( prePetList, self.PlayersPets );
end--MiniPet:HasMiniPetListChanged()

--Check if a player is in a battleground
function MiniPet.IsInBattleground()
    if( not MINIPET_BATTLEGROUNDS ) then
        local LBZ = LibStub( "LibBabble-Zone-3.0" ):GetLookupTable();

        MINIPET_BATTLEGROUNDS = table.concat( {
                LBZ["Warsong Gulch"],
                LBZ["Arathi Basin"],
                LBZ["Alterac Valley"],
                LBZ["Eye of the Storm"],
            }, "|" );
    end--if
    return MINIPET_BATTLEGROUNDS:find( GetRealZoneText() ) ~= nil;
end--MiniPet.IsInBattleground()

function MiniPet.HasQuestAndNotComplete( questID )
    local numEntries, _ = GetNumQuestLogEntries();

    if( numEntries > 0 ) then
        for id = 1, numEntries do
            local _, _, _, _, _, _, complete, _ = GetQuestLogTitle( id );
            
            if( ( not complete ) and ( LibLordFarlander.GetItemIDFromLink( GetQuestLink( id ) ) == questID ) ) then
                return true;
            end--if
        end--for
    end--if
    return false;
end--Minipet.GetQuestID( questName )
---Help Functions--
-------------------

----------------------
--+Message Functions--
--Give the player a reminder.
function MiniPet:WarnPlayer()
    local charDB = self.db.profile;

    if( ( not ( self.IsInBattleground() and charDB.BattleGroundCheck ) ) and ( not self.PlayerDied ) and ( not self.BoardedTaxi ) ) then--BG + option check
        self.FlashButton = charDB.GiveButtonWarning;
        if( charDB.GiveTextWarning ) then
            if( #self.PlayersPets > 0 ) then    --No use if player isn't cool and gots no pets!
                self.wprint( L["ALERT: Time to summon your minipet!"] );
            end--if
        end--if
        if( charDB.Button.AutoHideButton and ( not self.button:IsVisible() ) ) then
            self.button:ShowEx();
        end--if
    end--if
end--MiniPet:WarnPlayer()

--Shows a warning in the middle of the screen.
function MiniPet.wprint( msg )
    MinipetTextFrameText:SetTextColor( 1.0, 0.1, 0.1 );
    MinipetTextFrameText:SetText( msg );
    MinipetTextFrame.startTime = GetTime();
    MinipetTextFrame:Show();
end--MiniPet:wprint( msg )
---Message Functions
----------------------

-----------------
--+UI Functions--
function MiniPet:SetGameTooltips()
    local mpButton = self.button;

    if( not self.keyBoundMode ) then
        local pet = self.CurrentPet;
        local mpTooltip = mpButton.tooltip;
    
        mpTooltip:SetOwner( mpButton, "RIGHT" );
        if( pet == nil ) then
            mpTooltip:SetText( L["Get a minipet for some company!"] );
        else
            mpTooltip:SetFromListEntry( pet );
        end--if
    end--if
    LibStub( "LibLordFarlander-UI-2.0" ).AfterSetTooltip( mpButton );
end--MiniPet:SetGameTooltips()

function MiniPet:SetButton( pet )
    local mpButton = self.button;

    if( not InCombatLockdown() ) then
        if( pet == nil ) then
            mpButton.cooldown:Hide();
            mpButton:SetAttribute( "type1", "item" );
            mpButton:SetAttribute( "*bag1", nil );
            mpButton:SetAttribute( "*slot1", nil );
            mpButton:SetAttribute( "*item1", nil );

            mpButton:SetImage( "Interface\\AddOns\\MiniPet\\Img\\MiniPet" );

            self:dprint( "No pets" );
        else
            mpButton:SetFromListEntry( pet, not LibLordFarlander.CompareTables( self.LastPet, pet ) );
        end--if
        self.LastPet = pet;
    end--if
end--MiniPet:SetButton( pet )
---UI Functions--
-----------------
